#pragma once
#include "stdafx.h"
#include "TranslateFileInfo.h"
#include <WinInet.h>

#define MAX_PROGRESS_VALUE	100
#define NOTIFY_PROGRESS_MIN_TICKET 500

IMPLEMENT_SERIAL(TranslateFileInfo, CObject, 1)

TranslateFileInfo::TranslateFileInfo()
{
	Clear();
}
void TranslateFileInfo::Clear()
{
	m_pCallBack = NULL;
	m_nOsType = 0;
	m_nSqliteSeq = 0;
	m_TidRequestFileTranslate = 0;
    m_nFileSeq = 0;
	m_bUpload = FALSE;
	m_nJobIndex = -1;
	m_Status = sInvalid;
	m_nCurPos = 0;
	m_nTotalSize = 0;
	m_nLastProgress = 0;
	m_nLastTickCout = 0;
	m_nSqliteSeq = -1;
	m_transferResult = eUnknown;
}

TransferError TranslateFileInfo::ConvertTranslateError(DWORD dwLastError)
{
	TransferError eTransfer = eUnknown;
	switch (dwLastError)
	{
	case ERROR_SUCCESS:
		eTransfer = eSuccess;
		break;
	case ERROR_CANCELLED:
		eTransfer = eUserCancel;
		break;
	case ERROR_DISK_FULL:
		eTransfer = eNoEnoughDisk;
		break;
	default:
		if (dwLastError >= INTERNET_ERROR_BASE && dwLastError <= INTERNET_ERROR_LAST)
		{
			eTransfer = eNetwork;
			break;
		}
		else if(ERROR_INVALID_ADDRESS == dwLastError
			//CNTalkDownloadFileJob::_CheckInternetBuffer, maybe download file after 24 hours or can not find the files
			|| ERROR_LOST_WRITEBEHIND_DATA_NETWORK_SERVER_ERROR == dwLastError  
			//CChatDlg::OnJobEnd for strErrorInfo is not Empty, maybe server shutdown?
			)
		{
			eTransfer = eServer;
			break;
		}
		else
		{
			eTransfer = eUnknown;
		}
	}
	return eTransfer;
}

TranslateFileInfo::TranslateFileInfo(AttachFileInfo& fileInfo, BOOL bUpload)
{
	Clear();

	m_fileInfo = fileInfo;
	m_bUpload = bUpload;
	ResetProgressInfo();
}

TranslateFileInfo::TranslateFileInfo(AttachFileInfo& fileInfo, const std::vector<wstring>& vecReceiverIds,BOOL bUpload)
{
	Clear();
	m_fileInfo = fileInfo;
	m_vecReceiverIds = vecReceiverIds;
	m_bUpload = bUpload;

	ResetProgressInfo();
}

TranslateFileInfo::~TranslateFileInfo()
{

}

void TranslateFileInfo::ResetProgressInfo()
{
	m_nJobIndex = -1;
	//m_TidRequestFileTranslate = 0;
	m_nCurPos = 0;
	m_nTotalSize = 0;
	m_transferResult = eSuccess;
	m_Status = m_bUpload ? sUploadWaiting : sRecvWaiting;

	m_nLastProgress = 0;
	m_nLastTickCout = 0;
	if (m_pCallBack)
	{
		m_pCallBack->OnTransferProgress(*this, 0, TRUE);
	}
}

void TranslateFileInfo::SetTranslateCallback(ITranslateItemCallBack* pCallBack)
{
	m_pCallBack = pCallBack;
}

void TranslateFileInfo::SetResultByLastError(DWORD dwLastError)
{
	m_transferResult = ConvertTranslateError(dwLastError);
	switch (m_transferResult)
	{
	case eSuccess:
		m_Status = m_bUpload ? sUploadSuccess : sRecvSuccess;	
		break;
	default:
		m_Status = m_bUpload ? sUploadError : sRecvError;
		break;
	}
	if (m_pCallBack)
	{
		if (dwLastError == ERROR_SUCCESS)
		{
			m_pCallBack->OnTransferEnd(*this, m_Status);
		}
		else
		{
			m_pCallBack->OnTransferError(*this, m_transferResult);
		}
	}
}

void TranslateFileInfo::SetStatus(TransferStatus status)
{
	if (m_Status != status)
	{
		m_Status = status;
		if (m_pCallBack)
		{
			switch (m_Status)
			{
			case sUploading:
			case sReceiving:
				m_pCallBack->OnTransferStart(*this);
				break;
			case sUploadSuccess:
			case sUploadError:
			case sRecvSuccess:
			case sRecvError:
				m_pCallBack->OnTransferEnd(*this, m_Status);
				break;
			}
		}
	}
}

INT	TranslateFileInfo::GetProgress()
{
	LONGLONG nProgress = 0; //double
	ASSERT(m_nTotalSize >= 0);
	if (0 == m_nTotalSize)
	{
		ASSERT(0 == m_nCurPos); //waiting
		nProgress = 0;
	}
	else if(m_nTotalSize == m_nCurPos)
	{
		nProgress = MAX_PROGRESS_VALUE;	//success
	}
	else
	{
		//LONG will overflow after multiply by 100 if file size big than 2G/100
		nProgress = (LONGLONG)m_nCurPos * MAX_PROGRESS_VALUE / m_nTotalSize;  
	}
	ASSERT(nProgress >= 0);
	ASSERT(nProgress <= MAX_PROGRESS_VALUE);

	return (INT)nProgress;
}

void TranslateFileInfo::SetProgress(LONG nCurPos, LONG nTotalSize, BOOL bCalcSize)
{
	ASSERT(nCurPos >= 0);
	ASSERT(nTotalSize >= 0);
	m_nCurPos = nCurPos;
	m_nTotalSize = nTotalSize;
	INT nProgress = GetProgress();
	if (m_pCallBack)
	{
		DWORD nCurTickCount = GetTickCount();
		if ((m_nLastProgress != nProgress && nCurTickCount - m_nLastTickCout >= NOTIFY_PROGRESS_MIN_TICKET)
			|| MAX_PROGRESS_VALUE == nProgress 
			|| bCalcSize)
		{
			m_nLastTickCout = nCurTickCount;
			m_nLastProgress = nProgress;
			m_pCallBack->OnTransferProgress(*this, nProgress, bCalcSize);
		}
	}
}

void TranslateFileInfo::InPut( CArchive &archive )
{
	if(m_Status == 3)
	{
		ASSERT(FALSE);
	}
	CString Temp;
	archive>>Temp;
	m_fileInfo.downloadUrl = Temp;
	archive>>Temp;
	m_fileInfo.fileName= Temp;
	archive>>m_fileInfo.fileSize;
	archive>>Temp;
	m_fileInfo.flepath = Temp;

	
	archive>>m_bUpload;
	archive>>m_nJobIndex;
	
	archive>>m_TidRequestFileTranslate;
	archive>>m_tmTime;
	archive>>m_tmSendNotify;
	archive>>m_strSenderID;
	archive>>m_tmFailTime;	

	archive>>m_nOsType;
	archive>>m_strName;
	archive>>m_strNickName;
	int nStatus; 
	archive>>nStatus;
	m_Status = (TransferStatus)nStatus;
	archive>>m_nCurPos;		
	archive>>m_nTotalSize;

	archive>>m_nLastProgress;
	archive>>m_nSqliteSeq;
	archive>>m_nLastTickCout;
}

void TranslateFileInfo::OutPut( CArchive &archive )
{
	if(m_Status == 3)
	{
		ASSERT(FALSE);
	}
	archive<<CString(m_fileInfo.downloadUrl.c_str());
	archive<<CString(m_fileInfo.fileName.c_str());
	archive<<m_fileInfo.fileSize;
	archive<<CString(m_fileInfo.flepath.c_str());


	archive<<m_bUpload;
	archive<<m_nJobIndex;

	archive<<m_TidRequestFileTranslate;
	archive<<m_tmTime;
	archive<<m_tmSendNotify;
	archive<<m_strSenderID;
	archive<<m_tmFailTime;	

	archive<<m_nOsType;
	archive<<m_strName;
	archive<<m_strNickName;
	archive<<m_Status;
	archive<<m_nCurPos;		
	archive<<m_nTotalSize;

	archive<<m_nLastProgress;
	archive<<m_nSqliteSeq;
	archive<<m_nLastTickCout;
}

void TranslateFileInfo::Serialize( CArchive& archive )
{
	if(archive.IsStoring())
	{
		OutPut(archive);
	}
	else
	{
		InPut(archive);
	}
}
